@@ -14,14 +14,15 @@
 #    under the License.
 #
 
-import mock
+import copy
+from unittest import mock
 
 from vmware_nsxlib.tests.unit.v3 import nsxlib_testcase
 from vmware_nsxlib.tests.unit.v3.policy import test_resources
 from vmware_nsxlib.v3 import exceptions as nsxlib_exc
+from vmware_nsxlib.v3 import nsx_constants
 from vmware_nsxlib.v3.policy import constants
 from vmware_nsxlib.v3.policy import lb_defs
-
 TEST_TENANT = 'test'
 
 
@@ -930,8 +931,24 @@ class TestPolicyLBService(test_resources.NsxPolicyLibTestCase):
             self.resourceApi.get_statistics(obj_id, tenant=TEST_TENANT)
             expected_def = lb_defs.LBServiceStatisticsDef(
                 lb_service_id=obj_id,
+                realtime=False,
                 tenant=TEST_TENANT)
             self.assert_called_with_def(api_call, expected_def)
+            self.assertEqual('%s/lb-services/%s/statistics/',
+                             expected_def.path_pattern)
+
+    def test_get_statistics_realtime(self):
+        obj_id = '111'
+        with mock.patch.object(self.policy_api, "get") as api_call:
+            self.resourceApi.get_statistics(obj_id, realtime=True,
+                                            tenant=TEST_TENANT)
+            expected_def = lb_defs.LBServiceStatisticsDef(
+                lb_service_id=obj_id,
+                realtime=True,
+                tenant=TEST_TENANT)
+            self.assert_called_with_def(api_call, expected_def)
+            self.assertEqual('%s/lb-services/%s/statistics?source=realtime',
+                             expected_def.path_pattern)
 
     def test_get_virtual_server_status(self):
         obj_id = '111'
@@ -1121,6 +1138,52 @@ class TestPolicyLBVirtualServer(test_resources.NsxPolicyLibTestCase):
                 tenant=TEST_TENANT)
             self.assert_called_with_def(update_call, expected_def)
 
+    def test_update_log_parameters(self):
+        obj_id = '111'
+        name = 'new name'
+        description = 'new desc'
+        vs_name = 'name-name'
+        with self.mock_get(obj_id, vs_name), \
+            self.mock_create_update() as update_call:
+            self.resourceApi.update(obj_id,
+                                    name=name,
+                                    description=description,
+                                    tenant=TEST_TENANT,
+                                    access_log_enabled=True,
+                                    log_significant_event_only=True)
+            expected_def = lb_defs.LBVirtualServerDef(
+                nsx_version=nsx_constants.NSX_VERSION_3_0_0,
+                virtual_server_id=obj_id, name=name,
+                description=description,
+                tenant=TEST_TENANT, access_log_enabled=True,
+                log_significant_event_only=True)
+            self.assert_called_with_def(update_call, expected_def)
+
+    def test_log_parameters_for_version(self):
+        obj_id = '111'
+        name = 'new name'
+        description = 'new desc'
+
+        expected_def = lb_defs.LBVirtualServerDef(
+            nsx_version=nsx_constants.NSX_VERSION_2_5_0,
+            virtual_server_id=obj_id, name=name,
+            description=description,
+            tenant=TEST_TENANT, access_log_enabled=True,
+            log_significant_event_only=True)
+        self.assertFalse('access_log_enabled' in expected_def.get_obj_dict())
+        self.assertFalse('log_significant_event_only' in
+                         expected_def.get_obj_dict())
+
+        expected_def = lb_defs.LBVirtualServerDef(
+            nsx_version=nsx_constants.NSX_VERSION_3_0_0,
+            virtual_server_id=obj_id, name=name,
+            description=description,
+            tenant=TEST_TENANT, access_log_enabled=True,
+            log_significant_event_only=True)
+        self.assertTrue('access_log_enabled' in expected_def.get_obj_dict())
+        self.assertTrue('log_significant_event_only' in
+                        expected_def.get_obj_dict())
+
     def test_non_partial_update(self):
         obj_id = '111'
         vs_name = 'name-name'
@@ -1473,6 +1536,8 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
                 algorithm=algorithm,
                 member_group=member_group,
                 snat_translation=snat_translation,
+                tcp_multiplexing_enabled=True,
+                tcp_multiplexing_number=10,
                 tenant=TEST_TENANT)
             expected_def = lb_defs.LBPoolDef(
                 lb_pool_id=obj_id,
@@ -1483,6 +1548,8 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
                 algorithm=algorithm,
                 member_group=member_group,
                 snat_translation=snat_translation,
+                tcp_multiplexing_enabled=True,
+                tcp_multiplexing_number=10,
                 tenant=TEST_TENANT)
             self.assert_called_with_def(api_call, expected_def)
             self.assertEqual(obj_id, result)
@@ -1618,6 +1685,8 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
                                     algorithm=algorithm,
                                     member_group=member_group,
                                     snat_translation=snat_translation,
+                                    tcp_multiplexing_enabled=True,
+                                    tcp_multiplexing_number=10,
                                     tenant=TEST_TENANT)
             expected_def = lb_defs.LBPoolDef(
                 lb_pool_id=obj_id,
@@ -1628,6 +1697,8 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
                 algorithm=algorithm,
                 member_group=member_group,
                 snat_translation=snat_translation,
+                tcp_multiplexing_enabled=True,
+                tcp_multiplexing_number=10,
                 tenant=TEST_TENANT)
             self.assert_called_with_def(update_call, expected_def)
 
@@ -1727,7 +1798,8 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
         ip_address = '1.1.1.1'
         port = '80'
         new_name = 'mem1'
-        member = {'ip_address': ip_address, 'port': port}
+        member = {'ip_address': ip_address, 'port': port,
+                  'backup_member': True}
         with mock.patch.object(self.policy_api, "get",
                                return_value={'id': obj_id,
                                              'members': [member]}), \
@@ -1735,11 +1807,13 @@ class TestPolicyLBPoolApi(test_resources.NsxPolicyLibTestCase):
                               "create_or_update") as update_call:
             self.resourceApi.update_pool_member(
                 obj_id, ip_address, port=port, display_name=new_name,
-                tenant=TEST_TENANT)
-            member['display_name'] = new_name
+                backup_member=False, tenant=TEST_TENANT)
+            new_member = copy.copy(member)
+            new_member['display_name'] = new_name
+            new_member['backup_member'] = False
             expected_def = lb_defs.LBPoolDef(
                 lb_pool_id=obj_id,
-                members=[member],
+                members=[new_member],
                 tenant=TEST_TENANT)
             self.assert_called_with_def(update_call, expected_def)
 
