@@ -23,6 +23,9 @@ import itertools
 import logging
 import re
 import time
+from urllib import parse as urlparse
+
+import urllib3
 
 import eventlet
 from eventlet import greenpool
@@ -32,9 +35,6 @@ from oslo_log import log
 from oslo_service import loopingcall
 import requests
 from requests import adapters
-import six
-import six.moves.urllib.parse as urlparse
-import urllib3
 
 from vmware_nsxlib._i18n import _
 from vmware_nsxlib.v3 import client as nsx_client
@@ -53,8 +53,7 @@ logging.getLogger(
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractHTTPProvider(object):
+class AbstractHTTPProvider(object, metaclass=abc.ABCMeta):
     """Interface for providers of HTTP connections.
 
     which are responsible for creating and validating connections
@@ -243,6 +242,12 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
         # Add allow-overwrite if configured
         if allow_overwrite_header:
             session.default_headers['X-Allow-Overwrite'] = 'true'
+
+        if session.cert_provider:
+            # Session create will fail with cert provider
+            LOG.debug("Skipping session create with client certificate auth")
+            return
+
         # Perform the initial session create and get the relevant jsessionid &
         # X-XSRF-TOKEN for future requests
         req_data = ''
@@ -255,12 +260,15 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
                 bearer_token = token_provider.get_header_value(token_value)
                 token_header = {"Authorization": bearer_token}
                 session.default_headers.update(token_header)
-                req_headers.update(token_header)
+                # Don't call /api/session/create when using
+                # JWT Token Based Principal Identity auth scheme
+                LOG.debug("Skipping session create with JWT based auth")
+                return
             except exceptions.BadJSONWebTokenProviderRequest as e:
                 LOG.error("Session create failed for endpoint %s due to "
                           "error in retrieving JSON Web Token: %s",
                           provider.url, e)
-        elif not session.cert_provider:
+        else:
             # With client certificate authentication, username and password
             # may not be provided.
             # If provided, backend treats these credentials as authentication
@@ -451,18 +459,21 @@ class ClusteredAPI(object):
                  keepalive_interval=33,
                  api_rate_limit=None,
                  api_rate_mode=None,
-                 api_log_mode=None):
+                 api_log_mode=None,
+                 enable_health_check=True):
 
         self._http_provider = http_provider
         self._keepalive_interval = keepalive_interval
         self._print_keepalive = 0
         self._silent = False
         self._api_call_collectors = []
+        self._enable_health_check = enable_health_check
 
         def _init_cluster(*args, **kwargs):
             self._init_endpoints(providers, min_conns_per_pool,
                                  max_conns_per_pool, api_rate_limit,
-                                 api_rate_mode, api_log_mode)
+                                 api_rate_mode, api_log_mode,
+                                 enable_health_check)
 
         _init_cluster()
 
@@ -476,7 +487,7 @@ class ClusteredAPI(object):
 
     def _init_endpoints(self, providers, min_conns_per_pool,
                         max_conns_per_pool, api_rate_limit, api_rate_mode,
-                        api_log_mode):
+                        api_log_mode, enable_health_check=True):
         LOG.debug("Initializing API endpoints")
 
         def _create_conn(p):
@@ -516,30 +527,35 @@ class ClusteredAPI(object):
         # duck type to proxy http invocations
         for method in ClusteredAPI._HTTP_VERBS:
             setattr(self, method, self._proxy_stub(method))
-
-        conns = greenpool.GreenPool()
-        for endpoint in self._endpoints.values():
-            conns.spawn(self._validate, endpoint)
-        eventlet.sleep(0)
-        while conns.running():
-            if (self.health == ClusterHealth.GREEN or
-                self.health == ClusterHealth.ORANGE):
-                # only wait for 1 or more endpoints to reduce init time
-                break
-            eventlet.sleep(0.5)
-
-        if len(self._endpoints) > 1:
-            # We don't monitor connectivity when one endpoint is available,
-            # since there is no alternative to querying this single backend
-            # If endpoint was down, we can tolerate extra roundtrip to
-            # validate connectivity
+        # If health check is disabled, skip endpoint accessiblity check
+        # and health check loop. Set api health to GREEN.
+        if enable_health_check:
+            conns = greenpool.GreenPool()
             for endpoint in self._endpoints.values():
-                # dynamic loop for each endpoint to ensure connectivity
-                loop = loopingcall.DynamicLoopingCall(
-                    self._endpoint_keepalive, endpoint)
-                loop.start(initial_delay=self._keepalive_interval,
-                           periodic_interval_max=self._keepalive_interval,
-                           stop_on_exception=False)
+                conns.spawn(self._validate, endpoint)
+            eventlet.sleep(0)
+            while conns.running():
+                if (self.health == ClusterHealth.GREEN or
+                    self.health == ClusterHealth.ORANGE):
+                    # only wait for 1 or more endpoints to reduce init time
+                    break
+                eventlet.sleep(0.5)
+
+            if len(self._endpoints) > 1:
+                # We don't monitor connectivity when one endpoint is available,
+                # since there is no alternative to querying this single backend
+                # If endpoint was down, we can tolerate extra roundtrip to
+                # validate connectivity
+                for endpoint in self._endpoints.values():
+                    # dynamic loop for each endpoint to ensure connectivity
+                    loop = loopingcall.DynamicLoopingCall(
+                        self._endpoint_keepalive, endpoint)
+                    loop.start(initial_delay=self._keepalive_interval,
+                               periodic_interval_max=self._keepalive_interval,
+                               stop_on_exception=False)
+        else:
+            for endpoint in self._endpoints.values():
+                endpoint.set_state(EndpointState.UP)
 
         LOG.debug("Done initializing API endpoint(s). "
                   "API cluster health: %s", self.health)
@@ -766,10 +782,11 @@ class ClusteredAPI(object):
                         kwargs_copy = copy.copy(kwargs)
                         kwargs_copy['headers'] = utils.censor_headers(
                             kwargs_copy['headers'])
-                        LOG.debug("API cluster proxy %s %s to %s with %s. "
-                                  "Waited conn: %2.4f, rate: %2.4f",
-                                  proxy_for.upper(), uri, url, kwargs_copy,
-                                  conn_data.conn_wait, conn_data.rate_wait)
+                        LOG.debug("[%x] API cluster proxy %s %s to %s "
+                                  "with %s. Waited conn: %2.4f, rate: %2.4f",
+                                  id(conn), proxy_for.upper(), uri, url,
+                                  kwargs_copy, conn_data.conn_wait,
+                                  conn_data.rate_wait)
 
                     # call the actual connection method to do the
                     # http request/response over the wire
@@ -793,7 +810,7 @@ class ClusteredAPI(object):
 
                     return response
                 except Exception as e:
-                    LOG.warning("Request failed due to: %s", e)
+                    LOG.warning("[%x] Request failed due to: %s", id(conn), e)
                     exc_config = self.nsxlib_config.exception_config
                     if exc_config.should_ground_endpoint(e):
                         # consider endpoint inaccessible and move to next
@@ -830,7 +847,8 @@ class NSXClusteredAPI(ClusteredAPI):
             keepalive_interval=self.nsxlib_config.conn_idle_timeout,
             api_rate_limit=self.nsxlib_config.api_rate_limit_per_endpoint,
             api_rate_mode=self.nsxlib_config.api_rate_mode,
-            api_log_mode=self.nsxlib_config.api_log_mode)
+            api_log_mode=self.nsxlib_config.api_log_mode,
+            enable_health_check=self.nsxlib_config.enable_health_check)
 
         LOG.debug("Created NSX clustered API with '%s' "
                   "provider", self._http_provider.provider_id)
