@@ -79,6 +79,7 @@ class NsxPolicyLib(lib.NsxLibBase):
         self.tier0_route_map = core_resources.NsxPolicyTier0RouteMapApi(*args)
         self.tier0_prefix_list = core_resources.NsxPolicyTier0PrefixListApi(
             *args)
+        self.tier0_bgp = core_resources.NsxPolicyTier0BgpApi(*args)
         self.tier1 = core_resources.NsxPolicyTier1Api(*args)
         self.tier1_segment = core_resources.NsxPolicyTier1SegmentApi(*args)
         self.tier1_nat_rule = core_resources.NsxPolicyTier1NatRuleApi(
@@ -145,6 +146,9 @@ class NsxPolicyLib(lib.NsxLibBase):
         self.ipsec_vpn = ipsec_vpn_resources.NsxPolicyIpsecVpnApi(*args)
         self.global_config = core_resources.NsxPolicyGlobalConfig(*args)
 
+    def get_nsxlib_passthrough(self):
+        return self.nsx_api
+
     def get_version(self):
         """Get the NSX Policy manager version
 
