@@ -19,7 +19,6 @@ from distutils import version
 
 from oslo_log import log as logging
 import requests
-import six
 
 from vmware_nsxlib.v3 import exceptions
 from vmware_nsxlib.v3 import nsx_constants
@@ -77,8 +76,7 @@ TIER1_LOCALE_SERVICES_PATH_PATTERN = (TIER1S_PATH_PATTERN +
                                       "%s/locale-services/")
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ResourceDef(object):
+class ResourceDef(object, metaclass=abc.ABCMeta):
     def __init__(self, nsx_version=None, **kwargs):
         self.attrs = kwargs
 
@@ -566,6 +564,24 @@ class Tier1LocaleServiceDef(RouterLocaleServiceDef):
         return (TenantDef, Tier1Def)
 
 
+class Tier1MulticastDef(ResourceDef):
+
+    @property
+    def path_pattern(self):
+        return TIER1_LOCALE_SERVICES_PATH_PATTERN + "%s/multicast"
+
+    @property
+    def path_ids(self):
+        # 'Dummy' used since multicast is a sub-attribute of a Tier1 GW
+        # which is however set with its own API endpoint
+        return ('tenant', 'tier1_id', 'service_id', 'dummy')
+
+    def get_obj_dict(self):
+        body = super(Tier1MulticastDef, self).get_obj_dict()
+        self._set_attr_if_specified(body, 'enabled')
+        return body
+
+
 class Tier0InterfaceDef(ResourceDef):
 
     @staticmethod
@@ -649,7 +665,7 @@ class RouterNatRule(ResourceDef):
                                             'destination_network',
                                             'translated_network',
                                             'firewall_match',
-                                            'log',
+                                            'logging',
                                             'sequence_number',
                                             'enabled'])
         return body
@@ -967,7 +983,9 @@ class SegmentDef(BaseSegmentDef):
     def version_dependant_attr_map(self):
         return {'metadata_proxy_id': nsx_constants.NSX_VERSION_3_0_0,
                 'dhcp_server_config_id': nsx_constants.NSX_VERSION_3_0_0,
-                'admin_state': nsx_constants.NSX_VERSION_3_0_0}
+                'admin_state': nsx_constants.NSX_VERSION_3_0_0,
+                'unique_id': nsx_constants.NSX_VERSION_3_1_0,
+                'overlay_id': nsx_constants.NSX_VERSION_3_1_0}
 
     def get_obj_dict(self):
         body = super(SegmentDef, self).get_obj_dict()
@@ -996,7 +1014,8 @@ class SegmentDef(BaseSegmentDef):
             if self.get_attr('transport_zone_id'):
                 tz = TransportZoneDef(
                     tz_id=self.get_attr('transport_zone_id'),
-                    ep_id=constants.DEFAULT_ENFORCEMENT_POINT,
+                    ep_id=self.get_attr(
+                        'ep_id') or constants.DEFAULT_ENFORCEMENT_POINT,
                     tenant=self.get_tenant())
                 path = tz.get_resource_full_path()
             self._set_attr_if_specified(body, 'transport_zone_id',
@@ -1039,6 +1058,8 @@ class SegmentDef(BaseSegmentDef):
             self._set_attr_if_specified(body, 'admin_state',
                                         value=admin_state)
 
+        self._set_attr_if_supported(body, 'unique_id')
+        self._set_attr_if_supported(body, 'overlay_id')
         return body
 
 
@@ -1123,10 +1144,11 @@ class SegmentPortDef(ResourceDef):
 
     def get_obj_dict(self):
         body = super(SegmentPortDef, self).get_obj_dict()
-        address_bindings = self.get_attr('address_bindings')
-        if address_bindings:
-            body['address_bindings'] = [binding.get_obj_dict()
-                                        for binding in address_bindings]
+        if self.has_attr('address_bindings'):
+            address_bindings = self.get_attr('address_bindings')
+            if address_bindings is not None:
+                body['address_bindings'] = [binding.get_obj_dict()
+                                            for binding in address_bindings]
         if (self.has_attr('attachment_type') or self.has_attr('vif_id') or
             self.has_attr('hyperbus_mode')):
             if (not self.get_attr('attachment_type') and
@@ -2695,6 +2717,17 @@ class BgpRoutingConfigDef(ResourceDef):
         # This resource has no keys, since it is a single object
         return ('tenant', 'tier0_id', 'service_id', 'dummy')
 
+    def get_obj_dict(self):
+        body = super(BgpRoutingConfigDef, self).get_obj_dict()
+        self._set_attrs_if_specified(body, ['ecmp',
+                                            'enabled',
+                                            'graceful_restart_config',
+                                            'inter_sr_ibgp',
+                                            'local_as_num',
+                                            'multipath_relax',
+                                            'route_aggregations'])
+        return body
+
 
 class Tier0RouteRedistributionConfig(object):
 
