@@ -14,11 +14,9 @@
 #    under the License.
 #
 
-import abc
 import copy
 
 from oslo_log import log as logging
-import six
 
 from vmware_nsxlib._i18n import _
 from vmware_nsxlib.v3 import exceptions as nsxlib_exc
@@ -476,6 +474,8 @@ class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
                             tags=IGNORE, members=IGNORE, algorithm=IGNORE,
                             active_monitor_paths=IGNORE, member_group=IGNORE,
                             snat_translation=IGNORE,
+                            tcp_multiplexing_enabled=IGNORE,
+                            tcp_multiplexing_number=IGNORE,
                             tenant=constants.POLICY_INFRA_TENANT):
         lb_pool_id = self._init_obj_uuid(lb_pool_id)
         lb_pool_def = self._init_def(
@@ -488,6 +488,8 @@ class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
             algorithm=algorithm,
             member_group=member_group,
             snat_translation=snat_translation,
+            tcp_multiplexing_enabled=tcp_multiplexing_enabled,
+            tcp_multiplexing_number=tcp_multiplexing_number,
             tenant=tenant)
 
         self._create_or_store(lb_pool_def)
@@ -512,7 +514,8 @@ class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
     def update(self, lb_pool_id, name=IGNORE, description=IGNORE,
                tags=IGNORE, members=IGNORE, algorithm=IGNORE,
                active_monitor_paths=IGNORE, member_group=IGNORE,
-               snat_translation=IGNORE,
+               snat_translation=IGNORE, tcp_multiplexing_enabled=IGNORE,
+               tcp_multiplexing_number=IGNORE,
                tenant=constants.POLICY_INFRA_TENANT,
                allow_partial_updates=True):
         self._update(
@@ -525,6 +528,8 @@ class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
             algorithm=algorithm,
             member_group=member_group,
             snat_translation=snat_translation,
+            tcp_multiplexing_enabled=tcp_multiplexing_enabled,
+            tcp_multiplexing_number=tcp_multiplexing_number,
             tenant=tenant, allow_partial_updates=allow_partial_updates)
 
     def add_monitor_to_pool(self, lb_pool_id, active_monitor_paths,
@@ -587,7 +592,7 @@ class NsxPolicyLoadBalancerPoolApi(NsxPolicyResourceBase):
                 member_to_update[0]['weight'] = weight
             if admin_state:
                 member_to_update[0]['admin_state'] = admin_state
-            if backup_member:
+            if backup_member is not None:
                 member_to_update[0]['backup_member'] = backup_member
             self._update(lb_pool_id=lb_pool_id, members=lb_pool_members,
                          tenant=tenant)
@@ -720,12 +725,13 @@ class NsxPolicyLoadBalancerServiceApi(NsxPolicyResourceBase):
 
         _update()
 
-    def get_statistics(self, lb_service_id,
+    def get_statistics(self, lb_service_id, realtime=False,
                        tenant=constants.POLICY_INFRA_TENANT,
                        silent=False):
         lb_service_stats_def = (
             lb_defs.LBServiceStatisticsDef(
                 lb_service_id=lb_service_id,
+                realtime=realtime,
                 tenant=tenant))
         return self.policy_api.get(lb_service_stats_def, silent=silent)
 
@@ -790,7 +796,8 @@ class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
                             max_concurrent_connections=IGNORE,
                             access_list_control=IGNORE,
                             tenant=constants.POLICY_INFRA_TENANT,
-                            tags=IGNORE):
+                            tags=IGNORE, access_log_enabled=IGNORE,
+                            log_significant_event_only=IGNORE):
         virtual_server_id = self._init_obj_uuid(virtual_server_id)
         lbvs_def = self._init_def(
             virtual_server_id=virtual_server_id,
@@ -809,7 +816,9 @@ class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
             waf_profile_binding=waf_profile_binding,
             max_concurrent_connections=max_concurrent_connections,
             access_list_control=access_list_control,
-            tags=tags
+            tags=tags,
+            access_log_enabled=access_log_enabled,
+            log_significant_event_only=log_significant_event_only
         )
         self._create_or_store(lbvs_def)
         return virtual_server_id
@@ -843,7 +852,8 @@ class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
                access_list_control=IGNORE,
                tags=IGNORE,
                tenant=constants.POLICY_INFRA_TENANT,
-               allow_partial_updates=True):
+               allow_partial_updates=True, access_log_enabled=IGNORE,
+               log_significant_event_only=IGNORE):
 
         @utils.retry_upon_exception(
             nsxlib_exc.StaleRevision,
@@ -867,7 +877,9 @@ class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
                 max_concurrent_connections=max_concurrent_connections,
                 access_list_control=access_list_control,
                 tags=tags,
-                allow_partial_updates=allow_partial_updates)
+                allow_partial_updates=allow_partial_updates,
+                access_log_enabled=access_log_enabled,
+                log_significant_event_only=log_significant_event_only)
 
         _update()
 
@@ -1082,7 +1094,6 @@ class NsxPolicyLoadBalancerVirtualServerAPI(NsxPolicyResourceBase):
             sleep=sleep, max_attempts=max_attempts)
 
 
-@six.add_metaclass(abc.ABCMeta)
 class NsxPolicyLBMonitorProfileBase(NsxPolicyResourceBase):
     """NSX Policy LB monitor profile"""
 
